@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Command line utility to manage an index
- *                                                      Copyright (C) 2003-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2003-2006 Mikio Hirabayashi
  * This file is part of Estraier, a personal full-text search system.
  * Estraier is free software; you can redistribute it and/or modify it under the terms of the GNU
  * General Public License as published by the Free Software Foundation; either version 2 of the
@@ -89,14 +89,14 @@ void pdperror(const char *name);
 void printferror(const char *format, ...);
 void printfinfo(const char *format, ...);
 void relaxsystem(int inc);
-const char *datestr(int t);
+const char *datestr(time_t t);
 char *filetouri(const char *file);
 char *uritofile(const char *uri);
 int proclist(const char *name, const char *lfile);
 int procdir(const char *name, const char *dir);
 int indexdir(ODEUM *odeum, VILLA *mtdb, const char *name, const char *dir);
 int indexfile(ODEUM *odeum, VILLA *mtdb, const char *name, const char *file, const char *ruri);
-int ishot(VILLA *mtdb, const char *path, int mtime);
+int ishot(VILLA *mtdb, const char *path, time_t mtime);
 int procrelate(const char *name, const char *lfile);
 int procpurge(const char *name, const char *lfile);
 int procoptimize(const char *name, int small);
@@ -141,7 +141,7 @@ int main(int argc, char **argv){
   } else if(!strcmp(argv[1], "version") || !strcmp(argv[1], "--version")){
     printf("Estraier: a personal full-text search system version %s\n", _EST_VERSION);
     printf("Powered by QDBM version %s\n", dpversion);
-    printf("Copyright (c) 2003-2004 Mikio Hirabayashi\n");
+    printf("Copyright (C) 2003-2006 Mikio Hirabayashi\n");
     rv = 0;
   } else {
     usage();
@@ -645,15 +645,13 @@ void relaxsystem(int inc){
 
 
 /* get static string of the date in RFC822 format */
-const char *datestr(int t){
+const char *datestr(time_t t){
   static char buf[64], *wp;
   struct tm *tp, lt, gt;
-  time_t tt;
   int lag;
-  tt = (time_t)t;
-  if(!(tp = gmtime(&tt))) return "Thu, 01 Jan 1970 00:00:00 GMT";
+  if(!(tp = gmtime(&t))) return "Thu, 01 Jan 1970 00:00:00 GMT";
   gt = *tp;
-  if(!(tp = localtime(&tt))) return "Thu, 01 Jan 1970 00:00:00 GMT";
+  if(!(tp = localtime(&t))) return "Thu, 01 Jan 1970 00:00:00 GMT";
   lt = *tp;
   lag = (lt.tm_hour * 60 + lt.tm_min) - (gt.tm_hour * 60 + gt.tm_min);
   if(lt.tm_year > gt.tm_year){
@@ -868,7 +866,8 @@ int indexfile(ODEUM *odeum, VILLA *mtdb, const char *name, const char *file, con
   static int cnt = 0;
   char *buf, *uri;
   const char *cmd, *date, *title;
-  int size, mtime, wnum, bnum, err;
+  int size, wnum, bnum, err;
+  time_t mtime;
   ODDOC *doc;
   if(!cbfilestat(file, NULL, &size, &mtime)){
     printfinfo("%s: file does not exist", file);
@@ -990,7 +989,7 @@ int indexfile(ODEUM *odeum, VILLA *mtdb, const char *name, const char *file, con
     }
     docsettattrs(doc);
     if(odput(odeum, doc, wmax, TRUE) &&
-       vlput(mtdb, file, -1, (char *)&mtime, sizeof(int), VL_DOVER)){
+       vlput(mtdb, file, -1, (char *)&mtime, sizeof(time_t), VL_DOVER)){
       printfinfo("%s: registered: id=%d wnum=%d",
                  oddocuri(doc), oddocid(doc), cblistnum(oddocnwords(doc)));
       cnt++;
@@ -1024,12 +1023,12 @@ int indexfile(ODEUM *odeum, VILLA *mtdb, const char *name, const char *file, con
 
 
 /* check whetehr a document of the path is new or modified. */
-int ishot(VILLA *mtdb, const char *path, int mtime){
+int ishot(VILLA *mtdb, const char *path, time_t mtime){
   char *vbuf;
   int hot, vsiz;
   hot = TRUE;
   if((vbuf = vlget(mtdb, path, -1, &vsiz)) != NULL){
-    if(vsiz == sizeof(int) && mtime <= *(int *)vbuf) hot = FALSE;
+    if(vsiz == sizeof(int) && mtime <= *(time_t *)vbuf) hot = FALSE;
     free(vbuf);
   }
   return hot;
@@ -1451,7 +1450,7 @@ int procinform(const char *name){
   printf("all documents: %d\n", oddnum(odeum));
   printf("all words: %d\n", odwnum(odeum));
   printf("inode number: %d\n", odinode(odeum));
-  printf("modified time: %d\n", odmtime(odeum));
+  printf("modified time: %.0f\n", (double)odmtime(odeum));
   if(!odclose(odeum)){
     pdperror(name);
     return 1;
