@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Server to search an index
- *                                                      Copyright (C) 2003-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2003-2006 Mikio Hirabayashi
  * This file is part of Estraier, a personal full-text search system.
  * Estraier is free software; you can redistribute it and/or modify it under the terms of the GNU
  * General Public License as published by the Free Software Foundation; either version 2 of the
@@ -39,7 +39,7 @@ typedef struct {                         /* type of structure of a file */
   const char *name;                      /* file name */
   int isdir;                             /* whether to be a directory */
   char *type;                            /* media type */
-  int mtime;                             /* modified time */
+  time_t mtime;                          /* modified time */
   int size;                              /* size */
 } MYFILE;
 
@@ -182,7 +182,7 @@ int sockgetc(int fd);
 void showerror(int code, const char *msg, const char *extheads);
 int mywrite(int fd, const void *buf, int size);
 const char *urltofile(const char *url);
-const char *datestr(int t, int local);
+const char *datestr(time_t t, int local);
 const char *gettype(const char *path);
 void xmlprintf(const char *format, ...);
 void uriprint(const char *path, int mlen);
@@ -887,15 +887,13 @@ const char *urltofile(const char *url){
 
 
 /* get static string of the date in RFC822 format */
-const char *datestr(int t, int local){
+const char *datestr(time_t t, int local){
   static char buf[64], *wp;
   struct tm *tp, lt, gt;
-  time_t tt;
   int lag;
-  tt = (time_t)t;
-  if(!(tp = gmtime(&tt))) return "Thu, 01 Jan 1970 00:00:00 GMT";
+  if(!(tp = gmtime(&t))) return "Thu, 01 Jan 1970 00:00:00 GMT";
   gt = *tp;
-  if(!(tp = localtime(&tt))) return "Thu, 01 Jan 1970 00:00:00 GMT";
+  if(!(tp = localtime(&t))) return "Thu, 01 Jan 1970 00:00:00 GMT";
   lt = *tp;
   lag = (lt.tm_hour * 60 + lt.tm_min) - (gt.tm_hour * 60 + gt.tm_min);
   if(lt.tm_year > gt.tm_year){
@@ -1143,7 +1141,8 @@ void uriprint(const char *path, int mlen){
 void senddirview(const char *url, const char *query, int ims){
   const char *path, *name;
   char headbuf[SOCKBUFSIZ+32], cpath[ESTPATHBUFSIZ];
-  int i, isdir, size, mtime, fnum, stype;
+  int i, isdir, size, fnum, stype;
+  time_t mtime;
   CBLIST *list;
   MYFILE *files;
   path = urltofile(url);
@@ -1385,7 +1384,8 @@ int comparebysize(const void *a, const void *b){
 void sendasis(const char *url, int ims){
   const char *path;
   char headbuf[SOCKBUFSIZ+32], buf[SOCKBUFSIZ];
-  int isdir, size, mtime, rfd, len;
+  int isdir, size, rfd, len;
+  time_t mtime;
   path = urltofile(url);
   if(!cbfilestat(path, &isdir, &size, &mtime)){
     if(errno == EACCES){
