@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * Implementation of Depot
- *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -14,6 +14,8 @@
  *************************************************************************************************/
 
 
+#define QDBM_INTERNAL  1
+
 #include "depot.h"
 #include "myconf.h"
 
@@ -21,7 +23,7 @@
 #define DP_MAGICNUMB   "[DEPOT]\n\f"     /* magic number on environments of big endian */
 #define DP_MAGICNUML   "[depot]\n\f"     /* magic number on environments of little endian */
 #define DP_HEADSIZ     48                /* size of the reagion of the header */
-#define DP_LIBVEROFF   12                /* offset of the region for flags */
+#define DP_LIBVEROFF   12                /* offset of the region for the library version */
 #define DP_FLAGSOFF    16                /* offset of the region for flags */
 #define DP_FSIZOFF     24                /* offset of the region for the file size */
 #define DP_BNUMOFF     32                /* offset of the region for the bucket number */
@@ -30,7 +32,8 @@
 #define DP_FBPOOLSIZ   16                /* size of free block pool */
 #define DP_ENTBUFSIZ   128               /* size of the entity buffer */
 #define DP_STKBUFSIZ   256               /* size of the stack key buffer */
-#define DP_WRTBUFSIZ   2048              /* size of the writing buffer */
+#define DP_WRTBUFSIZ   8192              /* size of the writing buffer */
+#define DP_FSBLKSIZ    4096              /* size of a block of the file system */
 #define DP_TMPFSUF     MYEXTSTR "dptmp"  /* suffix of a temporary file */
 #define DP_OPTBLOAD    0.25              /* ratio of bucket loading at optimization */
 #define DP_OPTRUNIT    256               /* number of records in a process of optimization */
@@ -68,7 +71,7 @@ static int dpfirsthash(const char *kbuf, int ksiz);
 static int dpsecondhash(const char *kbuf, int ksiz);
 static int dpthirdhash(const char *kbuf, int ksiz);
 static int dpgetprime(int num);
-static int dppadsize(DEPOT *depot, int vsiz);
+static int dppadsize(DEPOT *depot, int ksiz, int vsiz);
 static int dprecsize(int *head);
 static int dprechead(DEPOT *depot, int off, int *head, char *ebuf, int *eep);
 static char *dpreckey(DEPOT *depot, int off, int *head);
@@ -127,8 +130,9 @@ const char *dperrmsg(int ecode){
 /* Get a database handle. */
 DEPOT *dpopen(const char *name, int omode, int bnum){
   char hbuf[DP_HEADSIZ], *map, c, *tname;
-  int i, mode, fd, inode, mtime, fsiz, rnum, msiz, *fbpool;
+  int i, mode, fd, inode, fsiz, rnum, msiz, *fbpool;
   struct stat sbuf;
+  time_t mtime;
   DEPOT *depot;
   assert(name);
   mode = O_RDONLY;
@@ -209,7 +213,7 @@ DEPOT *dpopen(const char *name, int omode, int bnum){
   }
   if((dpbigendian() ? memcmp(hbuf, DP_MAGICNUMB, strlen(DP_MAGICNUMB)) != 0 :
       memcmp(hbuf, DP_MAGICNUML, strlen(DP_MAGICNUML)) != 0) ||
-     memcmp(hbuf + DP_FSIZOFF, &fsiz, sizeof(int)) != 0){
+     *((int *)(hbuf + DP_FSIZOFF)) != fsiz){
     close(fd);
     dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
     return NULL;
@@ -976,7 +980,7 @@ int dpinode(DEPOT *depot){
 
 
 /* Get the last modified time of a database. */
-int dpmtime(DEPOT *depot){
+time_t dpmtime(DEPOT *depot){
   assert(depot);
   return depot->mtime;
 }
@@ -1245,6 +1249,108 @@ int dpprimenum(int num){
 }
 
 
+/* Retrieve a record directly from a database file. */
+char *dpsnaffle(const char *name, const char* kbuf, int ksiz, int *sp){
+  char hbuf[DP_HEADSIZ], *map, *vbuf, *tkbuf;
+  int fd, fsiz, bnum, msiz, *buckets, hash, thash, head[DP_RHNUM], off, vsiz, tksiz, kcmp;
+  struct stat sbuf;
+  assert(name && kbuf);
+  if(ksiz < 0) ksiz = strlen(kbuf);
+  if((fd = open(name, O_RDONLY, DP_FILEMODE)) == -1){
+    dpecodeset(DP_EOPEN, __FILE__, __LINE__);
+    return NULL;
+  }
+  if(fstat(fd, &sbuf) == -1 || !S_ISREG(sbuf.st_mode)){
+    close(fd);
+    dpecodeset(DP_ESTAT, __FILE__, __LINE__);
+    return NULL;
+  }
+  fsiz = sbuf.st_size;
+  if(!dpseekread(fd, 0, hbuf, DP_HEADSIZ)){
+    close(fd);
+    dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
+    return NULL;
+  }
+  if(dpbigendian() ? memcmp(hbuf, DP_MAGICNUMB, strlen(DP_MAGICNUMB)) != 0 :
+     memcmp(hbuf, DP_MAGICNUML, strlen(DP_MAGICNUML)) != 0){
+    close(fd);
+    dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
+    return NULL;
+  }
+  bnum = *((int *)(hbuf + DP_BNUMOFF));
+  if(bnum < 1 || fsiz < DP_HEADSIZ + bnum * sizeof(int)){
+    close(fd);
+    dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
+    return NULL;
+  }
+  msiz = DP_HEADSIZ + bnum * sizeof(int);
+  map = mmap(0, msiz, PROT_READ, MAP_SHARED, fd, 0);
+  if(map == MAP_FAILED){
+    close(fd);
+    dpecodeset(DP_EMAP, __FILE__, __LINE__);
+    return NULL;
+  }
+  buckets = (int *)(map + DP_HEADSIZ);
+  vbuf = NULL;
+  vsiz = 0;
+  hash = dpsecondhash(kbuf, ksiz);
+  off = buckets[dpfirsthash(kbuf, ksiz)%bnum];
+  while(off != 0){
+    if(!dpseekread(fd, off, head, DP_RHNUM * sizeof(int))) break;
+    if(head[DP_RHIKSIZ] < 0 || head[DP_RHIVSIZ] < 0 || head[DP_RHIPSIZ] < 0 ||
+       head[DP_RHILEFT] < 0 || head[DP_RHIRIGHT] < 0){
+      dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
+      break;
+    }
+    thash = head[DP_RHIHASH];
+    if(hash > thash){
+      off = head[DP_RHILEFT];
+    } else if(hash < thash){
+      off = head[DP_RHIRIGHT];
+    } else {
+      tksiz = head[DP_RHIKSIZ];
+      if(!(tkbuf = malloc(tksiz + 1))){
+        dpecodeset(DP_EALLOC, __FILE__, __LINE__);
+        break;
+      }
+      if(!dpseekread(fd, off + DP_RHNUM * sizeof(int), tkbuf, tksiz)){
+        free(tkbuf);
+        break;
+      }
+      tkbuf[tksiz] = '\0';
+      kcmp = dpkeycmp(kbuf, ksiz, tkbuf, tksiz);
+      free(tkbuf);
+      if(kcmp > 0){
+        off = head[DP_RHILEFT];
+      } else if(kcmp < 0){
+        off = head[DP_RHIRIGHT];
+      } else {
+        vsiz = head[DP_RHIVSIZ];
+        if(!(vbuf = malloc(vsiz + 1))){
+          dpecodeset(DP_EALLOC, __FILE__, __LINE__);
+          break;
+        }
+        if(!dpseekread(fd, off + DP_RHNUM * sizeof(int) + head[DP_RHIKSIZ], vbuf, vsiz)){
+          free(vbuf);
+          vbuf = NULL;
+          break;
+        }
+        vbuf[vsiz] = '\0';
+        break;
+      }
+    }
+  }
+  if(vbuf){
+    if(sp) *sp = vsiz;
+  } else {
+    dpecodeset(DP_ENOITEM, __FILE__, __LINE__);
+  }
+  munmap(map, msiz);
+  close(fd);
+  return vbuf;
+}
+
+
 
 /*************************************************************************************************
  * features for experts
@@ -1252,7 +1358,7 @@ int dpprimenum(int num){
 
 
 /* Name of the operating system. */
-char *dpsysname = _QDBM_SYSNAME;
+const char *dpsysname = _QDBM_SYSNAME;
 
 
 /* File descriptor for debugging output. */
@@ -1317,7 +1423,10 @@ int dpmemsync(DEPOT *depot){
 /* Get flags of a database. */
 int dpgetflags(DEPOT *depot){
   assert(depot);
-  return *(depot->map + DP_FLAGSOFF);
+  return (*(unsigned char *)(depot->map + DP_FLAGSOFF + 0) << 0) +
+    (*(unsigned char *)(depot->map + DP_FLAGSOFF + 1) << 8) +
+    (*(unsigned char *)(depot->map + DP_FLAGSOFF + 2) << 16) +
+    (*(unsigned char *)(depot->map + DP_FLAGSOFF + 3) << 24);
 }
 
 
@@ -1328,7 +1437,10 @@ int dpsetflags(DEPOT *depot, int flags){
     dpecodeset(DP_EMODE, __FILE__, __LINE__);
     return FALSE;
   }
-  *(depot->map + DP_FLAGSOFF) = flags;
+  *(depot->map + DP_FLAGSOFF + 0) = (flags >> 0) & 0xff;
+  *(depot->map + DP_FLAGSOFF + 1) = (flags >> 8) & 0xff;
+  *(depot->map + DP_FLAGSOFF + 2) = (flags >> 16) & 0xff;
+  *(depot->map + DP_FLAGSOFF + 3) = (flags >> 24) & 0xff;
   return TRUE;
 }
 
@@ -1612,15 +1724,25 @@ static int dpgetprime(int num){
 /* Get the padding size of a record.
    `vsiz' specifies the size of the value of a record.
    The return value is the padding size of a record. */
-static int dppadsize(DEPOT *depot, int vsiz){
-  int mod, pad;
+static int dppadsize(DEPOT *depot, int ksiz, int vsiz){
+  int pad;
   assert(depot && vsiz >= 0);
   if(depot->align > 0){
-    mod = vsiz % depot->align;
-    return mod == 0 ? 0 : depot->align - mod;
+    return depot->align - (depot->fsiz + DP_RHNUM * sizeof(int) + ksiz + vsiz) % depot->align;
   } else if(depot->align < 0){
     pad = (int)(vsiz * (2.0 / (1 << -(depot->align))));
-    return pad >= DP_RHNUM * sizeof(int) ? pad : DP_RHNUM * sizeof(int);
+    if(vsiz + pad >= DP_FSBLKSIZ){
+      if(vsiz <= DP_FSBLKSIZ) pad = 0;
+      if(depot->fsiz % DP_FSBLKSIZ == 0){
+        return (pad / DP_FSBLKSIZ) * DP_FSBLKSIZ + DP_FSBLKSIZ -
+          (depot->fsiz + DP_RHNUM * sizeof(int) + ksiz + vsiz) % DP_FSBLKSIZ;
+      } else {
+        return (pad / (DP_FSBLKSIZ / 2)) * (DP_FSBLKSIZ / 2) + (DP_FSBLKSIZ / 2) -
+          (depot->fsiz + DP_RHNUM * sizeof(int) + ksiz + vsiz) % (DP_FSBLKSIZ / 2);
+      }
+    } else {
+      return pad >= DP_RHNUM * sizeof(int) ? pad : DP_RHNUM * sizeof(int);
+    }
   }
   return 0;
 }
@@ -1654,7 +1776,7 @@ static int dprechead(DEPOT *depot, int off, int *head, char *ebuf, int *eep){
       *eep = TRUE;
       if(!dpseekread(depot->fd, off, ebuf, DP_ENTBUFSIZ)) return FALSE;
       memcpy(head, ebuf, DP_RHNUM * sizeof(int));
-      if(head[DP_RHIKSIZ] < 0 || head[DP_RHIVSIZ] < 0 || DP_RHIPSIZ < 0 ||
+      if(head[DP_RHIKSIZ] < 0 || head[DP_RHIVSIZ] < 0 || head[DP_RHIPSIZ] < 0 ||
          head[DP_RHILEFT] < 0 || head[DP_RHIRIGHT] < 0){
         dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
         return FALSE;
@@ -1663,7 +1785,7 @@ static int dprechead(DEPOT *depot, int off, int *head, char *ebuf, int *eep){
     }
   }
   if(!dpseekread(depot->fd, off, head, DP_RHNUM * sizeof(int))) return FALSE;
-  if(head[DP_RHIKSIZ] < 0 || head[DP_RHIVSIZ] < 0 || DP_RHIPSIZ < 0 ||
+  if(head[DP_RHIKSIZ] < 0 || head[DP_RHIVSIZ] < 0 || head[DP_RHIPSIZ] < 0 ||
      head[DP_RHILEFT] < 0 || head[DP_RHIRIGHT] < 0){
     dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
     return FALSE;
@@ -1888,7 +2010,7 @@ static int dprecappend(DEPOT *depot, const char *kbuf, int ksiz, const char *vbu
   char ebuf[DP_WRTBUFSIZ], *hbuf;
   int head[DP_RHNUM], asiz, psiz, off;
   assert(depot && kbuf && ksiz >= 0 && vbuf && vsiz >= 0);
-  psiz = dppadsize(depot, vsiz);
+  psiz = dppadsize(depot, ksiz, vsiz);
   head[DP_RHIFLAGS] = 0;
   head[DP_RHIHASH] = hash;
   head[DP_RHIKSIZ] = ksiz;
