@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The basic API of QDBM
- *                                                      Copyright (C) 2000-2005 Mikio Hirabayashi
+ *                                                      Copyright (C) 2000-2006 Mikio Hirabayashi
  * This file is part of QDBM, Quick Database Manager.
  * QDBM is free software; you can redistribute it and/or modify it under the terms of the GNU
  * Lesser General Public License as published by the Free Software Foundation; either version
@@ -23,6 +23,14 @@ extern "C" {
 
 
 #include <stdlib.h>
+#include <time.h>
+
+
+#if defined(_MSC_VER) && !defined(QDBM_INTERNAL) && !defined(QDBM_STATIC)
+#define MYEXTERN extern __declspec(dllimport)
+#else
+#define MYEXTERN extern
+#endif
 
 
 
@@ -33,9 +41,9 @@ extern "C" {
 
 typedef struct {                         /* type of structure for a database handle */
   char *name;                            /* name of the database file */
-  int wmode;                             /* whether writable or not */
+  int wmode;                             /* whether to be writable */
   int inode;                             /* inode of the database file */
-  int mtime;                             /* last modified time of the database */
+  time_t mtime;                          /* last modified time of the database */
   int fd;                                /* file descriptor of the database file */
   int fsiz;                              /* size of the database file */
   char *map;                             /* pointer to the mapped memory */
@@ -43,7 +51,7 @@ typedef struct {                         /* type of structure for a database han
   int *buckets;                          /* pointer to the bucket array */
   int bnum;                              /* number of the bucket array */
   int rnum;                              /* number of records */
-  int fatal;                             /* whether a fatal error occured or not */
+  int fatal;                             /* whether a fatal error occured */
   int ioff;                              /* offset of the iterator */
   int *fbpool;                           /* free block pool */
   int fbpsiz;                            /* size of the free block pool */
@@ -92,7 +100,7 @@ enum {                                   /* enumeration for write modes */
 
 
 /* String containing the version information. */
-extern const char *dpversion;
+MYEXTERN const char *dpversion;
 
 
 /* Last happened error code. */
@@ -329,7 +337,7 @@ int dpinode(DEPOT *depot);
 /* Get the last modified time of a database.
    `depot' specifies a database handle.
    The return value is the last modified time of the database. */
-int dpmtime(DEPOT *depot);
+time_t dpmtime(DEPOT *depot);
 
 
 /* Get the file descriptor of a database file.
@@ -394,26 +402,44 @@ int dpouterhash(const char *kbuf, int ksiz);
 int dpprimenum(int num);
 
 
+/* Retrieve a record directly from a database file.
+   `name' specifies the name of a database file.
+   `kbuf' specifies the pointer to the region of a key.
+   `ksiz' specifies the size of the region of the key.  If it is negative, the size is assigned
+   with `strlen(kbuf)'.
+   `sp' specifies the pointer to a variable to which the size of the region of the return
+   value is assigned.  If it is `NULL', it is not used.
+   If successful, the return value is the pointer to the region of the value of the
+   corresponding record, else, it is `NULL'.  `NULL' is returned when no record corresponds to
+   the specified key.
+   Because an additional zero code is appended at the end of the region of the return value,
+   the return value can be treated as a character string.  Because the region of the return
+   value is allocated with the `malloc' call, it should be released with the `free' call if it
+   is no longer in use.  Although this function can be used even while the database file is
+   locked by another process, it is not assured that recent updated is reflected. */
+char *dpsnaffle(const char *name, const char *kbuf, int ksiz, int *sp);
+
+
 
 /*************************************************************************************************
  * features for experts
  *************************************************************************************************/
 
 
-#define _QDBM_VERSION  "1.8.37"
-#define _QDBM_LIBVER   1109
+#define _QDBM_VERSION  "1.8.54"
+#define _QDBM_LIBVER   1215
 
 
 /* Name of the operating system. */
-extern char *dpsysname;
+MYEXTERN const char *dpsysname;
 
 
 /* File descriptor for debugging output. */
-extern int dpdbgfd;
+MYEXTERN int dpdbgfd;
 
 
 /* Whether this build is reentrant. */
-extern const int dpisreentrant;
+MYEXTERN const int dpisreentrant;
 
 
 /* Set the last happened error code.
@@ -441,12 +467,14 @@ int dpgetflags(DEPOT *depot);
 
 /* Set flags of a database.
    `depot' specifies a database handle connected as a writer.
-   `flags' specifies flags to set.
+   `flags' specifies flags to set.  Least ten bits are reserved for internal use.
    If successful, the return value is true, else, it is false. */
 int dpsetflags(DEPOT *depot, int flags);
 
 
 
+#undef MYEXTERN
+
 #if defined(__cplusplus)                 /* export for C++ */
 }
 #endif
